#include "TestScene.h"
#include <cmath>
#include <time.h>


USING_NS_CC;

CCScene* Test::scene()
{
    // 'scene' is an autorelease object
    CCScene *scene = CCScene::create();
    
    // 'layer' is an autorelease object
    Test *layer = Test::create();

    // add layer as a child to scene
    scene->addChild(layer);

    // return the scene
    return scene;
}

void Test::onEnter(){
    CCLayer::onEnter();
	this->setKeypadEnabled(true);
}

// on "init" you need to initialize your instance
bool Test::init()
{
	srand((unsigned int)time(NULL));
    //////////////////////////////
    // 1. super init first
    if ( !CCLayer::init() )
    {
        return false;
    }
    
    mVisibleSize = CCDirector::sharedDirector()->getVisibleSize();
	
	imgIndex = 0;
	powCounter = 10;

	this->setTouchEnabled(true);
    
	CCString* str = CCString::create("chr/tk01_damege0000.pvr");
	mBatchNode = CCSpriteBatchNode::create(str->getCString());
	this->addChild(mBatchNode);

	CCString* labelStr = CCString::createWithFormat( "Number of Sprite : %6d", imgIndex ) ;
	mNumberLabel = CCLabelTTF::create(labelStr->getCString(), "fonts/Marker Felt.ttf", 64.0f);
	mNumberLabel->setPosition(ccp(mNumberLabel->getContentSize().width *0.5f + 50, 200.0f));
	this->addChild(mNumberLabel);

    return true;
}

void Test::ccTouchesBegan(cocos2d::CCSet *pTouches, cocos2d::CCEvent *pEvent){
	CC_UNUSED_PARAM(pEvent);

	for (CCSetIterator it = pTouches->begin(); it != pTouches->end(); ++it)
    {
        CCTouch *touch = (CCTouch *)(*it);
    
		//上半分
		if(touch->getLocation().y > mVisibleSize.height / 2){
			//左半分
			if(touch->getLocation().x < mVisibleSize.width / 2){
				addSprite();
			}
			//右半分
			else{
				addSprite(10);
			}
		}
		//下半分
		else{
			removeSprite();
			addSprite(powCounter);
			powCounter *= 2;
		}
        CCString* labelStr = CCString::createWithFormat( "Number of Sprite : %6d", imgIndex ) ;
        mNumberLabel->setString(labelStr->getCString());
	}
}
void Test::ccTouchesMoved(cocos2d::CCSet *pTouches, cocos2d::CCEvent *pEvent){

}
void Test::ccTouchesCancelled(cocos2d::CCSet *pTouches, cocos2d::CCEvent *pEvent){

}
void Test::ccTouchesEnded(cocos2d::CCSet* pTouches, cocos2d::CCEvent* pEvent){

}


void Test::keyBackClicked(){
#if (CC_TARGET_PLATFORM == CC_PLATFORM_WINRT) || (CC_TARGET_PLATFORM == CC_PLATFORM_WP8)
	CCMessageBox("You pressed the close button. Windows Store Apps do not implement a close button.","Alert");
#else
    CCDirector::sharedDirector()->end();
#if (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
    exit(0);
#endif
#endif
}


void Test::addSprite(){
	
	CCSprite* pSprite = CCSprite::createWithTexture( mBatchNode->getTexture() );
	
	float rand_x = (rand() % (int)mVisibleSize.width) ;
	float rand_y = (rand() % (int)(mVisibleSize.height - 500) + 250) ;

	pSprite->setPosition( ccp(rand_x, rand_y) );
	mBatchNode->addChild(pSprite);
	imgIndex++;
}

void Test::addSprite(int addNum){
	for(int i=0;i<addNum;i++){
		addSprite();
	}
}

void Test::removeSprite(){
	mBatchNode->removeAllChildren();
	imgIndex = 0;
}