//
//  MainScene.cpp
//  SampleApp
//
//  Created by Hiroki Azumada on 2014/06/08.
//
//

#include "MainScene.h"
#include "TitleScene.h"
#include "Avator.h"
#include "Utils.h"
#include "App.h"
#include <SimpleAudioEngine.h>

static const std::string neko = "nekosample";


USING_NS_CC;


GameState::Mode				GameState::mode = GameState::Mode_SingleChar;
bool						GameState::isRandomScale = false;
GameState::TextureFormat	GameState::textureFormat = GameState::TF_PVRTC;
bool						GameState::useSpriteBatchNode = false;


CCScene* Main::createScene()
{
    // 'scene' is an autorelease object
    CCScene* scene = CCScene::create();
    
    // 'layer' is an autorelease object
    Main* layer = Main::create();
	layer->initContents();

    // add layer as a child to scene
    scene->addChild(layer);

    // return the scene
    return scene;
}

bool Main::init()
{
    //////////////////////////////
    // 1. super init first
    if ( !CCLayerGradient::initWithColor(
		ccc4(70,145,15,255),
		//Color4B(120,160,50,255),
		ccc4(170,220,120,255),
		ccp(0.0f, 1.0f)) )
    {
        return false;
    }
    return true;
}

void Main::initContents()
{
    CCSize visibleSize = CCDirector::sharedDirector()->getVisibleSize();
    CCPoint origin = CCDirector::sharedDirector()->getVisibleOrigin();

    CCMenuItemImage* closeItem = CCMenuItemImage::create(
		"Buttons/close.pvr",
		"Buttons/close_p.pvr",
		this, menu_selector(Main::menuCloseCallback));
    
	closeItem->setPosition(ccp(origin.x + visibleSize.width - closeItem->getContentSize().width/2 ,
                                origin.y + closeItem->getContentSize().height/2));

	float pmBtnY = 150;

	// +ボタン
	CCMenuItemImage* plusBtn = CCMenuItemImage::create(
		"Buttons/plus.pvr", "Buttons/plus_p.pvr",
		this, menu_selector(Main::menuPlusAvatorCallback));
	plusBtn->setPosition(
		origin.x + visibleSize.width / 6 * 5,
		pmBtnY
	);

	// -ボタン
	CCMenuItemImage* minusBtn = CCMenuItemImage::create(
		"Buttons/minus.pvr", "Buttons/minus_p.pvr",
		this, menu_selector(Main::menuMinusAvatorCallback));
	minusBtn->setPosition(
		origin.x + visibleSize.width / 6 * 1,
		pmBtnY
	);
	
	// +ボタン
	CCMenuItemImage* plusX10Btn = CCMenuItemImage::create(
		"Buttons/plus_x10.pvr", "Buttons/plus_x10_p.pvr",
		this, menu_selector(Main::menuPlus10AvatorCallback));
	plusX10Btn->setPosition(
		origin.x + visibleSize.width / 6 * 4,
		pmBtnY
	);

	// -ボタン
	CCMenuItemImage* minusX10Btn = CCMenuItemImage::create(
		"Buttons/minus_x10.pvr", "Buttons/minus_x10_p.pvr",
		this, menu_selector(Main::menuMinus10AvatorCallback));
	minusX10Btn->setPosition(
		origin.x + visibleSize.width / 6 * 2,
		pmBtnY
	);
	
	// 洋服ボタン
	CCMenuItemImage* wearBtn = CCMenuItemImage::create(
		"Buttons/wear.pvr", "Buttons/wear_p.pvr",
		this, menu_selector(Main::menuWearCallback));
	wearBtn->setPosition(
		origin.x + visibleSize.width / 6 * 3,
		pmBtnY
	);
	
    CCArray* buttons = new CCArray();
	buttons->addObject(closeItem);
	buttons->addObject(plusBtn);
	buttons->addObject(minusBtn);
	buttons->addObject(plusX10Btn);
	buttons->addObject(minusX10Btn);
	buttons->addObject(wearBtn);

	CCMenu* menu = CCMenu::createWithArray(buttons);
    menu->setPosition(CCPointZero);
    this->addChild(menu, 1);

    /////////////////////////////
	
	// アニメーションデータ、テクスチャ画像読み込み
	_clothes = Clothes_GlassAndWearAndHat;
	sbd::ResourceManager* resman = sbd::ResourceManager::getInstance();
	if (GameState::textureFormat == GameState::TF_PVRTC)
	{
		resman->addData("neko/nekosample.sbd", "neko/pvr", "pvr");
	}
	else
	{
		resman->addData("neko/nekosample.sbd", "neko/png32");
	}

	// 単一テクスチャモードでは、ネコ本体のテクスチャを参照するようセルの情報を書き換える
	if (GameState::mode == GameState::Mode_SingleChar || GameState::mode == GameState::Mode_SingleTexture)
	{
		resman->changeCell(neko, "megane", "neko1", 512, 96);
		resman->changeCell(neko, "wear", "neko1", 0, 512);
		resman->changeCell(neko, "hat", "neko1", 512, 512);
	}

	// アバター表示レイヤー
//	if (GameState::useSpriteBatchNode)
//	{
//		CCTexture2D* tex = resman->getFirstTexture(neko);
//		_playfield = CCSpriteBatchNode::createWithTexture(tex, 2000);
//	}
//	else
	{
		_playfield = CCNode::create();
	}

	this->addChild(_playfield);

	// アバター数の表示
    _countLabel = CCLabelTTF::create("a", "Arial", 32);	//24);
    _countLabel->setPosition(ccp(
		origin.x + visibleSize.width/2,
		origin.y + visibleSize.height - _countLabel->getContentSize().height - 100));
	updateCountLabel();
    this->addChild(_countLabel);

	// 効果音プリロード
	CocosDenshion::SimpleAudioEngine* audioEngien = CocosDenshion::SimpleAudioEngine::sharedEngine();
	audioEngien->preloadEffect(SE_PLUS);
	audioEngien->preloadEffect(SE_MINUS);
}

void Main::onEnter()
{
	CCLayerGradient::onEnter();

	CocosDenshion::SimpleAudioEngine* audioEngien = CocosDenshion::SimpleAudioEngine::sharedEngine();
	audioEngien->playBackgroundMusic(BGM_MAIN, true);

	if (GameState::isRandomScale)
	{
		for (int i = 0; i < 150; i++)
			createAvator();
	}
	else
	{
		createAvator();
	}
	
	this->scheduleUpdate();
}

void Main::update(float dt)
{
	CCObject* child;
	CCARRAY_FOREACH(_playfield->getChildren(), child)
	{
		sbd::Player* player = static_cast<sbd::Player*>(child);
		Avator* avator = static_cast<Avator*>(player->getUserObject());
		
		avator->update(dt);
	}
}

void Main::createAvator()
{
	sbd::Player* player = sbd::Player::create();
	player->useSpriteBatchNode(GameState::useSpriteBatchNode);
	player->setData(neko);

	float scale = 0.15f;
	if (GameState::isRandomScale)
	{
		scale = (float)_rand1.randomInt(20) * 0.01f + 0.05f;
	}
	player->setScale(scale);

    CCSize visibleSize = CCDirector::sharedDirector()->getVisibleSize();
    CCPoint origin = CCDirector::sharedDirector()->getVisibleOrigin();
	CCRect rect;
	rect.setRect(origin.x, origin.y, visibleSize.width, visibleSize.height);

	Avator* avator = Avator::create(rect, CCPoint(rect.getMidX(), rect.getMidY()));
	player->setUserObject(avator);
	avator->setFigure(player);

	int direction = _rand1.randomInt(180) * 2;
	avator->setDirection(direction);
	
	float speed = 1.0f + (float)(_rand1.randomInt(11) - 5) / 10.0f;
	avator->setSpeed(speed);
	
	int startFrameNo = _rand1.randomInt(30);
	
	int textureSetNo = 0;
	if (GameState::mode == GameState::Mode_SingleTexture || GameState::mode == GameState::Mode_MultiTextures)
	{
		textureSetNo = _rand2.randomInt(2);
	}

	player->play("anime_1", 0, startFrameNo, textureSetNo);
	player->setAllPartsTextureSet(textureSetNo);

	// 複数テクスチャモードでは洋服類もランダムで割り当てる
	if (GameState::mode == GameState::Mode_MultiTextures)
	{
		player->setPartTextureSet("megane", _rand2.randomInt(2));
		player->setPartTextureSet("wear", _rand2.randomInt(2));
		player->setPartTextureSet("hat", _rand2.randomInt(2));
	}

	changeClothes(player, _clothes);

#if 0
	auto particle = cocos2d::ParticleFire::createWithTotalParticles(100);
//	particle->setDuration(0.2);
//	particle->setPosition(Point(rect.getMidX(), rect.getMidY() - 20));
	particle->setLife(2);
	particle->setPosition(Point(-10, 300));
	player->addChild(particle);
#endif

	_rand1.randomInt(30);

	_playfield->addChild(player);

	updateCountLabel();
}

void Main::removeAvator()
{
	if (_playfield->getChildrenCount() <= 1) return;

	CCNode* last = (CCNode*)_playfield->getChildren()->objectAtIndex(_playfield->getChildrenCount()-1);
	_playfield->removeChild(last);

	updateCountLabel();
}

void Main::changeClothes(sbd::Player* player, Clothes clothes)
{
	bool glass = false;
	bool wear = false;
	bool hat = false;
	
	switch (clothes)
	{
	case Clothes_None:
		break;
	case Clothes_Glass:
		glass = true;
		break;
	case Clothes_GlassAndWear:
		glass = true;
		wear = true;
		break;
	case Clothes_GlassAndWearAndHat:
		glass = true;
		wear = true;
		hat = true;
		break;
	default:
		break;
	}

	player->setPartVisible("megane", glass);
	player->setPartVisible("wear", wear);
	player->setPartVisible("hat", hat);
}



void Main::menuPlusAvatorCallback(CCObject* pSender)
{
	CocosDenshion::SimpleAudioEngine* audioEngien = CocosDenshion::SimpleAudioEngine::sharedEngine();
	audioEngien->playEffect(SE_PLUS);

	createAvator();
}

void Main::menuPlus10AvatorCallback(CCObject* pSender)
{
	CocosDenshion::SimpleAudioEngine* audioEngien = CocosDenshion::SimpleAudioEngine::sharedEngine();
	audioEngien->playEffect(SE_PLUS);

	for (int i = 0; i < 10; i++)
		createAvator();
}

void Main::menuMinusAvatorCallback(CCObject* pSender)
{
	CocosDenshion::SimpleAudioEngine* audioEngien = CocosDenshion::SimpleAudioEngine::sharedEngine();
	audioEngien->playEffect(SE_MINUS);

	removeAvator();
}

void Main::menuMinus10AvatorCallback(CCObject* pSender)
{
	CocosDenshion::SimpleAudioEngine* audioEngien = CocosDenshion::SimpleAudioEngine::sharedEngine();
	audioEngien->playEffect(SE_MINUS);

	for (int i = 0; i < 10; i++)
		removeAvator();
}

void Main::menuWearCallback(CCObject* pSender)
{
	_clothes = (Clothes)(((int)_clothes + 1) % (int)Clothes_NumClothes);

	CCObject* child;
	CCARRAY_FOREACH(_playfield->getChildren(), child)
	{
		sbd::Player* player = (sbd::Player*)child;
		changeClothes(player, _clothes);
	}
}

void Main::menuCloseCallback(CCObject* pSender)
{
	CocosDenshion::SimpleAudioEngine* audioEngien = CocosDenshion::SimpleAudioEngine::sharedEngine();
	audioEngien->stopBackgroundMusic();

	CCScene* title = Title::createScene();
//	CCDirector::sharedDirector()->replaceScene(CCTransitionFade::create(0.2f, title));
	CCDirector::sharedDirector()->replaceScene(title);

	sbd::ResourceManager* resman = sbd::ResourceManager::getInstance();
	resman->removeAllData();
}

void Main::updateCountLabel()
{
	_countString = CCString::createWithFormat("Count: %d", (int)_playfield->getChildrenCount())->m_sString;
	_countLabel->setString(_countString.c_str());
}

