//
//  TitleScene.cpp
//  SampleApp
//
//  Created by Hiroki Azumada on 2014/06/08.
//
//

#include "TitleScene.h"
#include <SimpleAudioEngine.h>
#include "App.h"
#include "Utils.h"


USING_NS_CC;

CCScene* Title::createScene()
{
    // 'scene' is an autorelease object
    CCScene* scene = CCScene::create();
    
    // 'layer' is an autorelease object
    CCLayer* layer = Title::create();

    // add layer as a child to scene
    scene->addChild(layer);

    // return the scene
    return scene;
}

bool Title::init()
{
    if ( !CCLayer::init() )
    {
        return false;
    }
    
    CCSize visibleSize = CCDirector::sharedDirector()->getVisibleSize();
	CCLOG("visibleSize %f, %f", visibleSize.width, visibleSize.height);
    CCPoint origin = CCDirector::sharedDirector()->getVisibleOrigin();
	CCPoint center = CCPoint(origin.x + visibleSize.width / 2, origin.y + visibleSize.height / 2);


    CCMenuItemImage* closeItem = CCMenuItemImage::create(
		"Buttons/close.pvr",
		"Buttons/close_p.pvr",
		this,
		menu_selector(Title::menuCloseCallback));
    
	closeItem->setPosition(CCPoint(origin.x + visibleSize.width - closeItem->getContentSize().width/2 ,
                                origin.y + closeItem->getContentSize().height/2));
				
    CCArray* buttons = new CCArray();
	buttons->addObject(closeItem);
	
	// １キャラモード
	createMenuItem(buttons, ccp(center.x - 150, center.y + 130), "1 Tex", menu_selector(Title::menuStartCallback));
	// １キャラモード　バッチノード使用
	createMenuItem(buttons, ccp(center.x + 150, center.y + 130), "1 Tex (Batch)", menu_selector(Title::menuStartWithBatchCallback));
	// 単一テクスチャモード
	createMenuItem(buttons, ccp(center.x - 150, center.y +   0), "2 Tex", menu_selector(Title::menuStart2Callback));
	// 単一テクスチャモード　バッチノード使用
	createMenuItem(buttons, ccp(center.x + 150, center.y +   0), "2 Tex (Batch)", menu_selector(Title::menuStart2WithBatchCallback));
	// 複数テクスチャモード
	createMenuItem(buttons, ccp(center.x - 150, center.y - 130), "8 Tex", menu_selector(Title::menuStart3Callback));

	CCMenu* menu = CCMenu::createWithArray(buttons);
    menu->setPosition(CCPointZero);
    this->addChild(menu);


	// PNG/PVRTC選択トグル
	{
		unsigned int initialIndex = GameState::textureFormat == GameState::TF_PNG ? 1 : 0;
		CCMenu* toggleMenu = createToggleButton("Buttons/normal.pvr", "Buttons/check.pvr", this, menu_selector(Title::menuTextureFormatToggleCallback), initialIndex);
		toggleMenu->setPosition(center.x - 80, center.y - 270);
		this->addChild(toggleMenu);

		CCLabelTTF* label = CCLabelTTF::create("PNG", "Arial", 24);
		label->setPosition(ccp(toggleMenu->getPositionX(), toggleMenu->getPositionY() + 40));
		this->addChild(label);
	}

	// ランダムスケール選択トグル
	{
		unsigned int initialIndex = GameState::isRandomScale ? 1 : 0;
		CCMenu* toggleMenu = createToggleButton("Buttons/normal.pvr", "Buttons/check.pvr", this, menu_selector(Title::menuRandomScaleToggleCallback), initialIndex);
		toggleMenu->setPosition(center.x + 80, center.y - 270);
		this->addChild(toggleMenu);

		CCLabelTTF* label = CCLabelTTF::create("RandScale", "Arial", 24);
		label->setPosition(ccp(toggleMenu->getPositionX(), toggleMenu->getPositionY() + 40));
		this->addChild(label);
	}


	// BGM再生
	CocosDenshion::SimpleAudioEngine* audioEngien = CocosDenshion::SimpleAudioEngine::sharedEngine();
	audioEngien->playBackgroundMusic(BGM_TITLE);
	// スタート音をプリロードしておく
	audioEngien->preloadEffect(SE_START);
	
    return true;
}


void Title::menuStartCallback(CCObject* pSender)
{
	changeScene(GameState::Mode_SingleChar);
}

void Title::menuStartWithBatchCallback(CCObject* pSender)
{
	changeScene(GameState::Mode_SingleChar, true);
}

void Title::menuStart2Callback(CCObject* pSender)
{
	changeScene(GameState::Mode_SingleTexture);
}

void Title::menuStart2WithBatchCallback(CCObject* pSender)
{
	changeScene(GameState::Mode_SingleTexture, true);
}

void Title::menuStart3Callback(CCObject* pSender)
{
	changeScene(GameState::Mode_MultiTextures);
}

void Title::menuTextureFormatToggleCallback(CCObject* pSender)
{
	int selectedIndex = dynamic_cast<CCMenuItemToggle*>(pSender)->getSelectedIndex();
	GameState::textureFormat = selectedIndex == 1 ? GameState::TF_PNG : GameState::TF_PVRTC;
}

void Title::menuRandomScaleToggleCallback(CCObject* pSender)
{
	int selectedIndex = dynamic_cast<CCMenuItemToggle*>(pSender)->getSelectedIndex();
	GameState::isRandomScale = selectedIndex == 1;
}

void Title::menuCloseCallback(CCObject* pSender)
{
#if (CC_TARGET_PLATFORM == CC_PLATFORM_WP8) || (CC_TARGET_PLATFORM == CC_PLATFORM_WINRT)
	MessageBox("You pressed the close button. Windows Store Apps do not implement a close button.","Alert");
    return;
#endif

    CCDirector:: sharedDirector()->end();

#if (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
    exit(0);
#endif
}


void Title::changeScene(GameState::Mode mode, bool useSpriteBatchNode)
{
	CocosDenshion::SimpleAudioEngine* audioEngien = CocosDenshion::SimpleAudioEngine::sharedEngine();
	audioEngien->playEffect(SE_START);
	audioEngien->stopBackgroundMusic();

	GameState::mode = mode;
	GameState::useSpriteBatchNode = useSpriteBatchNode;
	CCScene* scene = Main::createScene();
	CCDirector* director = CCDirector::sharedDirector();
	//director->replaceScene(CCTransitionFade::create(0.3, main));
	director->replaceScene(scene);
}

void Title::createMenuItem(cocos2d::CCArray* buttons, cocos2d::CCPoint position, const std::string& label, cocos2d::SEL_MenuHandler selector)
{
    CCMenuItemImage* menuItem = CCMenuItemImage::create("Buttons/start.pvr", "Buttons/startClicked.pvr", this, selector);
	menuItem->setScale(2.0f);
	menuItem->setPosition(position);
	buttons->addObject(menuItem);
	
	CCLabelTTF* menuLabel = CCLabelTTF::create(label.c_str(), "Arial", 24);
	menuLabel->setPosition(ccp(menuItem->getPositionX(), menuItem->getPositionY()));
	this->addChild(menuLabel, 1);
}

cocos2d::CCMenu* Title::createToggleButton(const std::string& offTexName, const std::string& onTexName, cocos2d::CCObject* target, cocos2d::SEL_MenuHandler selector, unsigned int initialIndex)
{
	CCSprite* off1 = CCSprite::create(offTexName.c_str());
	CCSprite* off2 = CCSprite::create(offTexName.c_str());
	off2->setColor(ccc3(200, 100, 100));
	CCMenuItemSprite* offMenuItem = CCMenuItemSprite::create(off1, off2, NULL);

	CCSprite* on1 = CCSprite::create(onTexName.c_str());
	CCSprite* on2 = CCSprite::create(onTexName.c_str());
	on2->setColor(ccc3(200, 100, 100));
	CCMenuItemSprite* onMenuItem = CCMenuItemSprite::create(on1, on2, NULL);

	CCMenuItemToggle* toggle = CCMenuItemToggle::createWithTarget(target, selector, offMenuItem, onMenuItem, NULL);
	toggle->setSelectedIndex(initialIndex);

	CCMenu* toggleMenu = CCMenu::create(toggle, NULL);
	return toggleMenu;
}

