#include "TestScene.h"
#include <cmath>
#include <time.h>

Scene* Test::createScene()
{
	// 'scene' is an autorelease object
	auto scene = Scene::create();

	// 'layer' is an autorelease object
	auto layer = Test::create();

	// add layer as a child to scene
	scene->addChild(layer);

	// return the scene
	return scene;
}

// on "init" you need to initialize your instance
bool Test::init()
{
	//////////////////////////////
	// 1. super init first
	if (!Layer::init())
	{
		return false;
	}

	visibleSize = Director::getInstance()->getVisibleSize();
	Vec2 origin = Director::getInstance()->getVisibleOrigin();

	this->scheduleUpdate();

	imgIndex = 0;
	powCounter = 10;

	this->setKeypadEnabled(true);

	CCString* labelStr = CCString::createWithFormat("Number of Sprite : %d", imgIndex);
	numberLabel = Label::create(labelStr->getCString(), "fonts/Marker Felt.ttf", 32.0f);
	numberLabel->setPosition(ccp(270, 500));
	this->addChild(numberLabel);

    //タッチリスナー
	auto listener1 = EventListenerTouchOneByOne::create();
	listener1->setSwallowTouches(true);

	listener1->onTouchBegan = [=](Touch* touch, Event* event) {
		//上半分
		if (touch->getLocation().y > visibleSize.height / 2){
			//左半分
			if (touch->getLocation().x < visibleSize.width / 2)
			{
				addSprite();
			}
			//右半分
			else{
				addSprite(10);
			}
		}
		//下半分
		else{
			removeSprite();
			addSprite(powCounter);
			powCounter *= 2;
		}

		numberLabel->setString(String::createWithFormat("Number of Sprite : %d", imgIndex)->getCString());
		return false;
	};

	listener1->onTouchMoved = [](Touch* touch, Event* event) {};

	listener1->onTouchEnded = [=](Touch* touch, Event* event) {};

	_eventDispatcher->addEventListenerWithSceneGraphPriority(listener1, this);
    
    // キーリスナー
    auto keyListener = EventListenerKeyboard::create();
	keyListener->onKeyPressed = CC_CALLBACK_2(Test::onKeyPressed, this);
	_eventDispatcher->addEventListenerWithSceneGraphPriority(keyListener, this);
    
	return true;
}

void Test::update(float delta){

}

void Test::exitApp()
{
#if (CC_TARGET_PLATFORM == CC_PLATFORM_WP8) || (CC_TARGET_PLATFORM == CC_PLATFORM_WINRT)
	MessageBox("You pressed the close button. Windows Store Apps do not implement a close button.","Alert");
    return;
#endif
    
    Director::getInstance()->end();
    
#if (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
    exit(0);
#endif
}

void Test::addSprite(){
	CCString* str;
	int index = imgIndex % 16;
	if (index < 10){
		str = CCString::createWithFormat("chr/tk01_damege000%d.pvr", index);
	}
	else{
		str = CCString::createWithFormat("chr/tk01_damege00%d.pvr", index);
	}
	Sprite* pSprite = Sprite::create(str->getCString());

	float rand_x = (rand() % (int)visibleSize.width);
	float rand_y = (rand() % (int)visibleSize.height);

	pSprite->setPosition(ccp(rand_x, rand_y));
	this->addChild(pSprite, -1);
	imgIndex++;
}

void Test::addSprite(int addNum){
	for (int i = 0; i<addNum; i++){
		addSprite();
	}
}

void Test::removeSprite(){

	this->removeAllChildren();
	imgIndex = 0;

	auto labelStr = StringUtils::format("Number of Sprite : %d", imgIndex);
	numberLabel = Label::create(labelStr.c_str(), "fonts/Marker Felt.ttf", 32.0f);
	numberLabel->setPosition(ccp(270, 500));
	this->addChild(numberLabel);
}

void Test::onKeyReleased(EventKeyboard::KeyCode keyCode, Event* event){
	if (keyCode == EventKeyboard::KeyCode::KEY_ESCAPE){
		exitApp();
	}
	else if (keyCode == EventKeyboard::KeyCode::KEY_MENU){
		removeSprite();
	}
}