//
//  Avator.cpp
//  SampleApp
//
//  Created by Hiroki Azumada on 2014/06/08.
//
//

#include "Avator.h"
#include "Utils.h"
#include "App.h"

#include "cocos2d.h"
#include <SimpleAudioEngine.h>

static const float PI = 3.14159265359;


Avator::Avator()
	: _action(Action::Stop)
	, _actionTime(0)
	, _direction(0)
	, _speed(1.0f)
	, _actionDelay(0)
	, _figure(nullptr)
{
}

Avator* Avator::create(const cocos2d::Rect& movabled, const cocos2d::Point& startPosition)
{
	Avator* obj = new Avator();
	if (obj)
	{
		obj->_position = startPosition;
		obj->_movabled = movabled;
		obj->autorelease();
	}
	return obj;
}

void Avator::setFigure(sbd::Player *figure)
{
	CC_SAFE_RELEASE(_figure);
	CC_SAFE_RETAIN(figure);
	_figure = figure;
}

void Avator::setDirection(int direction)
{
	_direction = direction;
	bool flipX = direction < 180;
	_figure->setFlippedX(flipX);
}

void Avator::update(float dt)
{
	if (!_figure) return;
	
	int dir = (_direction + 270) % 360;
	float r = (float)dir * (PI / 180.0f);
	float x = (float) std::cos(r);
	float y = (float)-std::sin(r);
	
	float s = dt / (1.0f / 60.0f);
	s *= _speed;
	x *= s;
	y *= s;

	_position.x += x;
	_position.y += y;

	if (_actionDelay <= 0)
	{
		if (!_movabled.containsPoint(_position))
		{
			auto newDir = (_direction + 100) % 360;
			setDirection(newDir);
			_actionDelay = 10;
		}
	}
	else
	{
		_actionDelay--;
	}
	
	_figure->setPosition(_position);
	_figure->setLocalZOrder(1000 - (int)_position.y);
}

