//
//  MainScene.cpp
//  SampleApp
//
//  Created by Hiroki Azumada on 2014/06/08.
//
//

#include "MainScene.h"
#include "TitleScene.h"
#include "Avator.h"
#include "Utils.h"
#include "App.h"
#include <SimpleAudioEngine.h>

static const std::string neko = "nekosample";


USING_NS_CC;


GameState::Mode				GameState::mode = GameState::Mode::SingleChar;
bool						GameState::isRandomScale = false;
GameState::TextureFormat	GameState::textureFormat = GameState::TextureFormat::PVRTC;
bool						GameState::useSpriteBatchNode = false;


Scene* Main::createScene()
{
    // 'scene' is an autorelease object
    auto scene = Scene::create();
    
    // 'layer' is an autorelease object
    auto layer = Main::create();
	layer->initContents();

    // add layer as a child to scene
    scene->addChild(layer);

    // return the scene
    return scene;
}

bool Main::init()
{
    //////////////////////////////
    // 1. super init first
    if ( !LayerGradient::initWithColor(
		Color4B(70,145,15,255),
		//Color4B(120,160,50,255),
		Color4B(170,220,120,255),
		Point(0.0f, 1.0f)) )
    {
        return false;
    }
    return true;
}

void Main::initContents()
{
    Size visibleSize = Director::getInstance()->getVisibleSize();
    Point origin = Director::getInstance()->getVisibleOrigin();

    auto closeItem = MenuItemImage::create(
		"Buttons/close.pvr",
		"Buttons/close_p.pvr",
		CC_CALLBACK_1(Main::menuCloseCallback, this));
    
	closeItem->setPosition(Point(origin.x + visibleSize.width - closeItem->getContentSize().width/2 ,
                                origin.y + closeItem->getContentSize().height/2));

	float pmBtnY = 150;

	// +ボタン
	auto plusBtn = MenuItemImage::create(
		"Buttons/plus.pvr", "Buttons/plus_p.pvr",
		CC_CALLBACK_1(Main::menuPlusAvatorCallback, this));
	plusBtn->setPosition(
		origin.x + visibleSize.width / 6 * 5,
		pmBtnY
	);

	// -ボタン
	auto minusBtn = MenuItemImage::create(
		"Buttons/minus.pvr", "Buttons/minus_p.pvr",
		CC_CALLBACK_1(Main::menuMinusAvatorCallback, this));
	minusBtn->setPosition(
		origin.x + visibleSize.width / 6 * 1,
		pmBtnY
	);
	
	// +ボタン
	auto plusX10Btn = MenuItemImage::create(
		"Buttons/plus_x10.pvr", "Buttons/plus_x10_p.pvr",
		CC_CALLBACK_1(Main::menuPlus10AvatorCallback, this));
	plusX10Btn->setPosition(
		origin.x + visibleSize.width / 6 * 4,
		pmBtnY
	);

	// -ボタン
	auto minusX10Btn = MenuItemImage::create(
		"Buttons/minus_x10.pvr", "Buttons/minus_x10_p.pvr",
		CC_CALLBACK_1(Main::menuMinus10AvatorCallback, this));
	minusX10Btn->setPosition(
		origin.x + visibleSize.width / 6 * 2,
		pmBtnY
	);
	
	// 洋服ボタン
	auto wearBtn = MenuItemImage::create(
		"Buttons/wear.pvr", "Buttons/wear_p.pvr",
		CC_CALLBACK_1(Main::menuWearCallback, this));
	wearBtn->setPosition(
		origin.x + visibleSize.width / 6 * 3,
		pmBtnY
	);
	
	Vector<MenuItem*> buttons;
	buttons.pushBack(closeItem);
	buttons.pushBack(plusBtn);
	buttons.pushBack(minusBtn);
	buttons.pushBack(plusX10Btn);
	buttons.pushBack(minusX10Btn);
	buttons.pushBack(wearBtn);

	auto menu = Menu::createWithArray(buttons);
    menu->setPosition(Point::ZERO);
    this->addChild(menu, 1);

    /////////////////////////////

	// アニメーションデータ、テクスチャ画像読み込み
	_clothes = Clothes::GlassAndWearAndHat;
	auto resman = sbd::ResourceManager::getInstance();
	if (GameState::textureFormat == GameState::TextureFormat::PVRTC)
	{
		resman->addData("neko/nekosample.sbd", "neko/pvr", "pvr");
	}
	else
	{
		resman->addData("neko/nekosample.sbd", "neko/png32");
	}

	// 単一テクスチャモードでは、ネコ本体のテクスチャを参照するようセルの情報を書き換える
	if (GameState::mode == GameState::Mode::SingleChar || GameState::mode == GameState::Mode::SingleTexture)
	{
		resman->changeCell(neko, "megane", "neko1", 512, 96);
		resman->changeCell(neko, "wear", "neko1", 0, 512);
		resman->changeCell(neko, "hat", "neko1", 512, 512);
	}

	// アバター表示レイヤー
	_playfield = Node::create();
	this->addChild(_playfield);

	// アバター数の表示
    _countLabel = LabelTTF::create("a", "Arial", 32);
    _countLabel->setPosition(Point(
		origin.x + visibleSize.width/2,
		origin.y + visibleSize.height - _countLabel->getContentSize().height - 100));
	updateCountLabel();
    this->addChild(_countLabel);

	// 効果音プリロード
	auto audioEngien = CocosDenshion::SimpleAudioEngine::getInstance();
	audioEngien->preloadEffect(SE_PLUS);
	audioEngien->preloadEffect(SE_MINUS);
}

void Main::onEnter()
{
	LayerGradient::onEnter();
	
	auto audioEngien = CocosDenshion::SimpleAudioEngine::getInstance();
	audioEngien->playBackgroundMusic(BGM_MAIN, true);

	if (GameState::isRandomScale)
	{
		for (int i = 0; i < 150; i++)
			createAvator();
	}
	else
	{
		createAvator();
	}
	
	this->scheduleUpdate();
}

void Main::update(float dt)
{
	for (auto child : _playfield->getChildren())
	{
		sbd::Player* player = static_cast<sbd::Player*>(child);
		Avator* avator = static_cast<Avator*>(player->getUserObject());
		
		avator->update(dt);
	}
}

void Main::createAvator()
{
	auto player = sbd::Player::create();
	player->useSpriteBatchNode(GameState::useSpriteBatchNode);
	player->setData(neko);
	
	float scale = 0.15f;
	if (GameState::isRandomScale)
	{
		scale = (float)_rand1.randomInt(20) * 0.01f + 0.05f;
	}
	player->setScale(scale);

    Size visibleSize = Director::getInstance()->getVisibleSize();
    Point origin = Director::getInstance()->getVisibleOrigin();
	Rect rect;
	rect.setRect(origin.x, origin.y, visibleSize.width, visibleSize.height);

	auto avator = Avator::create(rect, Point(rect.getMidX(), rect.getMidY()));
	player->setUserObject(avator);
	avator->setFigure(player);

	int direction = _rand1.randomInt(180) * 2;
	avator->setDirection(direction);
	
	float speed = 1.0f + (float)(_rand1.randomInt(11) - 5) / 10.f;
	avator->setSpeed(speed);
	
	int startFrameNo = _rand1.randomInt(30);
	
	int textureSetNo = 0;
	if (GameState::mode == GameState::Mode::SingleTexture || GameState::mode == GameState::Mode::MultiTextures)
	{
		textureSetNo = _rand2.randomInt(2);
	}

	player->play("anime_1", 0, startFrameNo, textureSetNo);
	player->setAllPartsTextureSet(textureSetNo);

	// 複数テクスチャモードでは洋服類もランダムで割り当てる
	if (GameState::mode == GameState::Mode::MultiTextures)
	{
		player->setPartTextureSet("megane", _rand2.randomInt(2));
		player->setPartTextureSet("wear", _rand2.randomInt(2));
		player->setPartTextureSet("hat", _rand2.randomInt(2));
	}

	changeClothes(player, _clothes);

#if 0
	auto particle = cocos2d::ParticleFire::createWithTotalParticles(100);
//	particle->setDuration(0.2);
//	particle->setPosition(Point(rect.getMidX(), rect.getMidY() - 20));
	particle->setLife(2);
	particle->setPosition(Point(-10, 300));
	player->addChild(particle);
#endif

	_rand1.randomInt(30);

	_playfield->addChild(player);

	updateCountLabel();
}

void Main::removeAvator()
{
	if (_playfield->getChildrenCount() <= 1) return;

	auto last = _playfield->getChildren().at(_playfield->getChildrenCount()-1);
	_playfield->removeChild(last);

	updateCountLabel();
}

void Main::changeClothes(sbd::Player* player, Clothes clothes)
{
	bool glass = false;
	bool wear = false;
	bool hat = false;
	
	switch (clothes)
	{
	case Clothes::None:
		break;
	case Clothes::Glass:
		glass = true;
		break;
	case Clothes::GlassAndWear:
		glass = true;
		wear = true;
		break;
	case Clothes::GlassAndWearAndHat:
		glass = true;
		wear = true;
		hat = true;
		break;
	default:
		break;
	}

	player->setPartVisible("megane", glass);
	player->setPartVisible("wear", wear);
	player->setPartVisible("hat", hat);
}


void Main::menuPlusAvatorCallback(Ref* pSender)
{
	auto audioEngien = CocosDenshion::SimpleAudioEngine::getInstance();
	audioEngien->playEffect(SE_PLUS);

	createAvator();
}

void Main::menuPlus10AvatorCallback(Ref* pSender)
{
	auto audioEngien = CocosDenshion::SimpleAudioEngine::getInstance();
	audioEngien->playEffect(SE_PLUS);

	for (int i = 0; i < 10; i++)
		createAvator();
}

void Main::menuMinusAvatorCallback(Ref* pSender)
{
	auto audioEngien = CocosDenshion::SimpleAudioEngine::getInstance();
	audioEngien->playEffect(SE_MINUS);

	removeAvator();
}

void Main::menuMinus10AvatorCallback(Ref* pSender)
{
	auto audioEngien = CocosDenshion::SimpleAudioEngine::getInstance();
	audioEngien->playEffect(SE_MINUS);

	for (int i = 0; i < 10; i++)
		removeAvator();
}

void Main::menuWearCallback(Ref* pSender)
{
	_clothes = (Clothes)(((int)_clothes + 1) % (int)Clothes::NumClothes);

	for (auto child : _playfield->getChildren())
	{
		sbd::Player* player = (sbd::Player*)child;
		changeClothes(player, _clothes);
	}
}

void Main::menuCloseCallback(Ref* pSender)
{
	this->unscheduleUpdate();
	this->removeAllChildrenWithCleanup(true);

	auto audioEngien = CocosDenshion::SimpleAudioEngine::getInstance();
	audioEngien->stopBackgroundMusic();

	auto title = Title::createScene();
//	Director::getInstance()->replaceScene(TransitionFade::create(0.2f, title));
	Director::getInstance()->replaceScene(title);

	auto resman = sbd::ResourceManager::getInstance();
	resman->removeAllData();
}

void Main::updateCountLabel()
{
	_countString = cocos2d::StringUtils::format("Count: %d", (int)_playfield->getChildrenCount());
	_countLabel->setString(_countString);
}

