//
//  SbdPlayerData.h
//

#ifndef SbdPlayerData_h
#define SbdPlayerData_h

namespace sbd
{

typedef short			ss_s16;
typedef unsigned short	ss_u16;
typedef unsigned int	ss_u32;
typedef int				ss_s32;
typedef int				ss_offset;


struct CellMap
{
	ss_offset	name;			// const char*
	ss_offset	imagePath;		// const char*
	ss_s16		index;
	ss_s16		reserved;
};


struct Cell
{
	ss_offset	name;			// const char*
	ss_offset	cellMap;		// const CellMap*
	ss_s16		indexInCellMap;
	ss_s16		x;
	ss_s16		y;
	ss_u16		width;
	ss_u16		height;
	ss_s16		reserved;
	float		pivotX;			// 0.0:left, 0.5:center, 1.0:right
	float		pivotY;			// 0.0:bottom, 0.5:center, 1.0:top
};


struct AnimationData
{
	ss_offset	name;			// const char*
	ss_offset	frameData;		// const ss_s16*
	ss_s16		numFrames;
	ss_s16		fps;
};


struct AnimationInitialData
{
	ss_s16		index;
	ss_u16		flags;
	ss_s16		cellIndex;
	ss_s16		positionX;
	ss_s16		positionY;
	ss_u16		opacity;
	float		anchorX;
	float		anchorY;
	float		rotation;
	float		scaleX;
	float		scaleY;
};


struct PartData
{
	ss_offset	name;			// const char*
	ss_s16		index;
	ss_s16		parentIndex;
	ss_s16		type;
	ss_s16		boundsType;
	ss_s16		alphaBlendType;	// BlendType
	ss_s16		reserved;
};


struct AnimePackData
{
	ss_offset	name;			// const char*
	ss_offset	parts;			// const PartData*
	ss_offset	animations;		// const AnimationData*
	ss_s16		numParts;
	ss_s16		numAnimations;
};


struct ProjectData
{
	ss_u32		dataId;
	ss_u32		version;
	ss_u32		flags;
	ss_offset	imageBaseDir;	// const char*
	ss_offset	cells;			// const Cell*
	ss_offset	animePacks;		// const AnimePackData*
	ss_s16		numCells;
	ss_s16		numAnimePacks;
};


};	// namespace sbd

#endif
