//
//  TitleScene.cpp
//  SampleApp
//
//  Created by Hiroki Azumada on 2014/06/08.
//
//

#include "TitleScene.h"
#include <SimpleAudioEngine.h>
#include "App.h"



USING_NS_CC;

Scene* Title::createScene()
{
    // 'scene' is an autorelease object
    auto scene = Scene::create();
    
    // 'layer' is an autorelease object
    auto layer = Title::create();

    // add layer as a child to scene
    scene->addChild(layer);

    // return the scene
    return scene;
}

bool Title::init()
{
    if ( !Layer::init() )
    {
        return false;
    }
    
    Size visibleSize = Director::getInstance()->getVisibleSize();
	log("visibleSize %f, %f", visibleSize.width, visibleSize.height);
    Point origin = Director::getInstance()->getVisibleOrigin();
	Point center = Point(origin.x + visibleSize.width / 2, origin.y + visibleSize.height / 2);


    auto closeItem = MenuItemImage::create(
		"Buttons/close.pvr", "Buttons/close_p.pvr",
		CC_CALLBACK_1(Title::menuCloseCallback, this));

	closeItem->setPosition(Point(origin.x + visibleSize.width - closeItem->getContentSize().width/2 ,
                                origin.y + closeItem->getContentSize().height/2));

	Vector<MenuItem*> buttons;
	buttons.pushBack(closeItem);

	// １キャラモード
	createMenuItem(buttons, Point(center.x - 150, center.y + 130), "1 Tex", CC_CALLBACK_1(Title::menuStartCallback, this));
	// １キャラモード　バッチノード使用
	createMenuItem(buttons, Point(center.x + 150, center.y + 130), "1 Tex (Batch)", CC_CALLBACK_1(Title::menuStartWithBatchCallback, this));
	// 単一テクスチャモード
	createMenuItem(buttons, Point(center.x - 150, center.y +   0), "2 Tex", CC_CALLBACK_1(Title::menuStart2Callback, this));
	// 単一テクスチャモード　バッチノード使用
	createMenuItem(buttons, Point(center.x + 150, center.y +   0), "2 Tex (Batch)", CC_CALLBACK_1(Title::menuStart2WithBatchCallback, this));
	// 複数テクスチャモード
	createMenuItem(buttons, Point(center.x - 150, center.y - 130), "8 Tex", CC_CALLBACK_1(Title::menuStart3Callback, this));

	Menu* menu = Menu::createWithArray(buttons);
    menu->setPosition(Point::ZERO);
    this->addChild(menu);

	// PNG/PVRTC選択トグル
	{
		unsigned int initialIndex = GameState::textureFormat == GameState::TextureFormat::PNG ? 1 : 0;
		Menu* toggleMenu = createToggleButton("Buttons/normal.pvr", "Buttons/check.pvr", CC_CALLBACK_1(Title::menuTextureFormatToggleCallback, this), initialIndex);
		toggleMenu->setPosition(center.x - 80, center.y - 270);
		this->addChild(toggleMenu);

		LabelTTF* label = LabelTTF::create("PNG", "Arial", 24);
		label->setPosition(toggleMenu->getPositionX(), toggleMenu->getPositionY() + 40);
		this->addChild(label);
	}

	// ランダムスケール選択トグル
	{
		unsigned int initialIndex = GameState::isRandomScale ? 1 : 0;
		Menu* toggleMenu = createToggleButton("Buttons/normal.pvr", "Buttons/check.pvr", CC_CALLBACK_1(Title::menuRandomScaleToggleCallback, this), initialIndex);
		toggleMenu->setPosition(center.x + 80, center.y - 270);
		this->addChild(toggleMenu);

		LabelTTF* label = LabelTTF::create("RandScale", "Arial", 24);
		label->setPosition(toggleMenu->getPositionX(), toggleMenu->getPositionY() + 40);
		this->addChild(label);
	}


	// BGM再生
	auto audioEngien = CocosDenshion::SimpleAudioEngine::getInstance();
	audioEngien->playBackgroundMusic(BGM_TITLE);
	// スタート音をプリロードしておく
	audioEngien->preloadEffect(SE_START);
	
    return true;
}


void Title::menuStartCallback(Ref* pSender)
{
	changeScene(GameState::Mode::SingleChar);
}

void Title::menuStartWithBatchCallback(Ref* pSender)
{
	changeScene(GameState::Mode::SingleChar, true);
}

void Title::menuStart2Callback(Ref* pSender)
{
	changeScene(GameState::Mode::SingleTexture);
}

void Title::menuStart2WithBatchCallback(Ref* pSender)
{
	changeScene(GameState::Mode::SingleTexture, true);
}

void Title::menuStart3Callback(Ref* pSender)
{
	changeScene(GameState::Mode::MultiTextures);
}

void Title::menuTextureFormatToggleCallback(Ref* pSender)
{
	int selectedIndex = dynamic_cast<MenuItemToggle*>(pSender)->getSelectedIndex();
	GameState::textureFormat = selectedIndex == 1 ? GameState::TextureFormat::PNG : GameState::TextureFormat::PVRTC;
}

void Title::menuRandomScaleToggleCallback(Ref* pSender)
{
	int selectedIndex = dynamic_cast<MenuItemToggle*>(pSender)->getSelectedIndex();
	GameState::isRandomScale = selectedIndex == 1;
}

void Title::menuCloseCallback(Ref* pSender)
{
#if (CC_TARGET_PLATFORM == CC_PLATFORM_WP8) || (CC_TARGET_PLATFORM == CC_PLATFORM_WINRT)
	MessageBox("You pressed the close button. Windows Store Apps do not implement a close button.","Alert");
    return;
#endif

    Director::getInstance()->end();

#if (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
    exit(0);
#endif
}


void Title::changeScene(GameState::Mode mode, bool useSpriteBatchNode)
{
	auto audioEngien = CocosDenshion::SimpleAudioEngine::getInstance();
	audioEngien->playEffect(SE_START);
	audioEngien->stopBackgroundMusic();

	GameState::mode = mode;
	GameState::useSpriteBatchNode = useSpriteBatchNode;
	auto scene = Main::createScene();
	auto director = Director::getInstance();
//	director->replaceScene(TransitionFade::create(0.3, main));
	director->replaceScene(scene);
}

void Title::createMenuItem(cocos2d::Vector<cocos2d::MenuItem*>& buttons, const cocos2d::Vec2& pos, const std::string& label, const cocos2d::ccMenuCallback& callback)
{
    auto menuItem = CCMenuItemImage::create("Buttons/start.pvr", "Buttons/startClicked.pvr", callback);
	menuItem->setScale(2.0f);
	menuItem->setPosition(pos);
	buttons.pushBack(menuItem);
	
	LabelTTF* menuLabel = LabelTTF::create(label.c_str(), "Arial", 24);
	menuLabel->setPosition(ccp(menuItem->getPositionX(), menuItem->getPositionY()));
	this->addChild(menuLabel, 1);
}

cocos2d::Menu* Title::createToggleButton(const std::string& offTexName, const std::string& onTexName, const cocos2d::ccMenuCallback& callback, unsigned int initialIndex)
{
	Sprite* off1 = Sprite::create(offTexName.c_str());
	Sprite* off2 = Sprite::create(offTexName.c_str());
	off2->setColor(Color3B(200, 100, 100));
	MenuItemSprite* offMenuItem = MenuItemSprite::create(off1, off2, NULL);

	Sprite* on1 = Sprite::create(onTexName.c_str());
	Sprite* on2 = Sprite::create(onTexName.c_str());
	on2->setColor(Color3B(200, 100, 100));
	MenuItemSprite* onMenuItem = MenuItemSprite::create(on1, on2, NULL);

	MenuItemToggle* toggle = MenuItemToggle::createWithCallback(callback, offMenuItem, onMenuItem, NULL);
	toggle->setSelectedIndex(initialIndex);

	Menu* toggleMenu = Menu::create(toggle, NULL);
	return toggleMenu;
}


