﻿using UnityEngine;
//using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;

public class MainScript : MonoBehaviour {

	public enum Clothes
	{
		None,
		Glass,
		GlassAndWear,
		GlassAndWearAndHat,
	}


	public GUIStyle closeButton = new GUIStyle();
	public GUIStyle plusButton = new GUIStyle();
	public GUIStyle plusX10Button = new GUIStyle();
	public GUIStyle minusButton = new GUIStyle();
	public GUIStyle minusX10Button = new GUIStyle();
	public GUIStyle wearButton = new GUIStyle();
	public GUIStyleState styleState = new GUIStyleState();
	public GUIStyle textStyle = new GUIStyle();

	public const string SE_PLUS = "191592__fins__menu-button";
	public const string SE_MINUS = "191593__fins__menu-button";

	private AudioSource _sePlus;
	private AudioSource _seMinus;

	private int _avatorCount;
	private List<SbdPlayer> _players = new List<SbdPlayer>(500);
	private Clothes _clothes;

	private FRandom _rand1 = new FRandom();
	private FRandom _rand2 = new FRandom();

	private float _accumDt;
	private float _frameRate;
	private uint _frames;


	void Awake() {
		Camera camera = GameObject.Find("Main Camera").GetComponent<Camera>();
		camera.orthographicSize = Screen.height/2;
	}


	void Start () {

		closeButton.normal.background = Resources.Load("Buttons/close") as Texture2D;
		closeButton.active.background = Resources.Load("Buttons/close_p") as Texture2D;

		plusButton.normal.background = Resources.Load("Buttons/plus") as Texture2D;
		plusButton.active.background = Resources.Load("Buttons/plus_p") as Texture2D;
		plusX10Button.normal.background = Resources.Load("Buttons/plus_x10") as Texture2D;
		plusX10Button.active.background = Resources.Load("Buttons/plus_x10_p") as Texture2D;
		minusButton.normal.background = Resources.Load("Buttons/minus") as Texture2D;
		minusButton.active.background = Resources.Load("Buttons/minus_p") as Texture2D;
		minusX10Button.normal.background = Resources.Load("Buttons/minus_x10") as Texture2D;
		minusX10Button.active.background = Resources.Load("Buttons/minus_x10_p") as Texture2D;

		wearButton.normal.background = Resources.Load("Buttons/wear") as Texture2D;
		wearButton.active.background = Resources.Load("Buttons/wear_p") as Texture2D;

		textStyle.fontSize = 24;
		styleState.textColor = Color.white;
		textStyle.normal = styleState;

		var audioSources = GetComponents<AudioSource>();
		_sePlus = audioSources[0];
		_seMinus = audioSources[1];

		// アニメーションデータ、テクスチャ画像読み込み
		_clothes = Clothes.GlassAndWearAndHat;
		var resman = ResourceManager.Instance();
		if (GameState.textureFormat == GameState.TextureFormat.PVRTC)
		{
			resman.AddData("neko/nekosample", "neko/pvr");
		}
		else
		{
			resman.AddData("neko/nekosample", "neko/png32");
		}

		if (GameState.mode == GameState.Mode.SingleChar || GameState.mode == GameState.Mode.SingleTexture)
		{
			resman.ChangeCell("neko/nekosample", "megane", "neko1", 512, 96);
			resman.ChangeCell("neko/nekosample", "wear", "neko1", 0, 512);
			resman.ChangeCell("neko/nekosample", "hat", "neko1", 512, 512);
		}

		if (GameState.withRandomScale)
		{
			for (int i = 0; i < 150; i++)
			{
				CreateAvator();
			}
		}
		else
		{
			CreateAvator();
		}

		_accumDt = 0;
		_frameRate = 0;
	}


	void Update () {
	}


	private const float StatsInterval = 0.5f;

	void OnGUI () {

		_frames++;
		_accumDt += Time.deltaTime;

		if (_accumDt > StatsInterval)
		{
			_frameRate = (float)_frames / _accumDt;
			_frames = 0;
			_accumDt = 0;
		}

		// fps
		var labelSize = new Vector2(70, 50);
		GUI.Label(new Rect(10, Screen.height - labelSize.y - 5, labelSize.x, labelSize.y), string.Format("{0:0.00}", _frameRate), textStyle);

		// avator count
		labelSize = new Vector2(70, 50);
		GUI.Label(new Rect((Screen.width - labelSize.x) / 2, 100, labelSize.x, labelSize.y), "Count: " + _avatorCount, textStyle);

		float centerY = Screen.height / 2;

		float buttonY = centerY + 200;

		var buttonSize = new Vector2(65, 65);
		float width = Screen.width;


		if (GUI.Button(new Rect(width / 6 * 5 - buttonSize.x/2, buttonY, buttonSize.x, buttonSize.y), "", plusButton))
		{
			_sePlus.PlayOneShot(_sePlus.clip);
			CreateAvator();
		}

		if (GUI.Button(new Rect(width / 6 * 1 - buttonSize.x/2, buttonY, buttonSize.x, buttonSize.y), "", minusButton))
		{
			_seMinus.PlayOneShot(_seMinus.clip);
			RemoveAvator();
		}
		
		if (GUI.Button(new Rect(width / 6 * 4 - buttonSize.x/2, buttonY, buttonSize.x, buttonSize.y), "", plusX10Button))
		{
			_sePlus.PlayOneShot(_sePlus.clip);
			for (int i = 0; i < 10; i++) CreateAvator();
		}

		if (GUI.Button(new Rect(width / 6 * 2 - buttonSize.x/2, buttonY, buttonSize.x, buttonSize.y), "", minusX10Button))
		{
			_seMinus.PlayOneShot(_seMinus.clip);
			for (int i = 0; i < 10; i++) RemoveAvator();
		}

		if (GUI.Button(new Rect(width / 6 * 3 - buttonSize.x/2, buttonY, buttonSize.x, buttonSize.y), "", wearButton))
		{
			_sePlus.PlayOneShot(_sePlus.clip);

			var no = ((int)_clothes + 1) % System.Enum.GetNames(typeof(Clothes)).Length;
			_clothes = (Clothes)no;

			ChangeClothes(_clothes);
		}


		var closeWidth = closeButton.normal.background.width;
		var closeHeight = closeButton.normal.background.height;
		
		if (GUI.Button(new Rect(Screen.width - closeWidth, Screen.height - closeHeight, closeWidth, closeHeight), "", closeButton))
		{
			ResourceManager.Instance().RemoveAllData();
			Application.LoadLevel("Title");
		}
	}


	void CreateAvator()
	{
		var player = SbdPlayer.Create("avator");
		player.SetData("neko/nekosample");

		float scale = 0.15f;
		//float scale = (float)_rand1.RandomInt(20) * 0.01f + 0.05f;
		if (GameState.withRandomScale)
		{
			scale = (float)_rand1.RandomInt(20) * 0.01f + 0.05f;
		}
		player.transform.localScale = new Vector3(scale, scale, scale);

		var avator = player.gameObject.AddComponent<Avatar>();
		avator.Initialize(new Rect(0, 0, Screen.width, Screen.height), new Vector2(0, 0));

		int direction = _rand1.RandomInt(180) * 2;
		avator.direction = direction;

		float speed = 1.0f + (float)(_rand1.RandomInt(11) - 5) / 10.0f;
		avator.speed = speed;

		int startFrameNo = _rand1.RandomInt(30);

		int textureSetNo = 0;
		if (GameState.mode == GameState.Mode.SingleTexture || GameState.mode == GameState.Mode.MultiTextures)
		{
			textureSetNo = _rand2.RandomInt(2);
		}

		player.Play("anime_1", 0, startFrameNo);
		player.SetAllPartsTextureSet(textureSetNo);

		// 複数テクスチャモードでは洋服類もランダムで割り当てる
		if (GameState.mode == GameState.Mode.MultiTextures)
		{
			player.SetPartTextureSet("megane", _rand2.RandomInt(2));
			player.SetPartTextureSet("wear", _rand2.RandomInt(2));
			player.SetPartTextureSet("hat", _rand2.RandomInt(2));
		}

		ChangeClothes(player, _clothes);

		_rand1.RandomInt(30);

		_players.Add(player);
		_avatorCount++;
	}


	void RemoveAvator()
	{
		if (_avatorCount <= 1) return;

		var removedIndex = _players.Count - 1;
		var player = _players[removedIndex];
		_players.RemoveAt(removedIndex);

		Destroy(player.gameObject);
		_avatorCount--;
	}


	void ChangeClothes(Clothes clothes)
	{
		foreach (var player in _players)
		{
			ChangeClothes(player, clothes);
		}
	}


	void ChangeClothes(SbdPlayer player, Clothes clothes)
	{
		bool glass = false;
		bool wear = false;
		bool hat = false;
		
		switch (clothes)
		{
		case Clothes.None:
			break;
		case Clothes.Glass:
			glass = true;
			break;
		case Clothes.GlassAndWear:
			glass = true;
			wear = true;
			break;
		case Clothes.GlassAndWearAndHat:
			glass = true;
			wear = true;
			hat = true;
			break;
		default:
			break;
		}

		player.SetPartVisible("megane", glass);
		player.SetPartVisible("wear", wear);
		player.SetPartVisible("hat", hat);
	}

}
